use core::arch::asm;

use lazy_static::lazy_static;
use spin::Mutex;
use uart_16550::SerialPort;

lazy_static! {
	pub static ref SERIAL1: Mutex<SerialPort> = {
		let mut serial_port = unsafe { SerialPort::new(0x3F8) };
		serial_port.init();
		Mutex::new(serial_port)
	};
}

#[doc(hidden)]
pub fn _print(args: core::fmt::Arguments<'_>) {
	use core::fmt::Write;

	// TODO: preserve previous interrupt state

	// Disable interrupts to prevent deadlocks
	// (we might get an interrupt while printing)
	unsafe {
		asm!("cli", options(preserves_flags, nostack));
	}

	SERIAL1
		.lock()
		.write_fmt(args)
		.expect("Printing to serial failed");

	unsafe {
		asm!("sti", options(preserves_flags, nostack));
	}
}

/// Prints to the host through the serial interface.
#[macro_export]
macro_rules! print {
	($($arg:tt)*) => {
		$crate::drivers::serial::_print(format_args!($($arg)*));
	};
}

/// Prints to the host through the serial interface, appending a newline.
#[macro_export]
macro_rules! println {
	() => ($crate::print!("\n"));
	($fmt:expr) => ($crate::print!(concat!($fmt, "\n")));
	($fmt:expr, $($arg:tt)*) => ($crate::print!(
		concat!($fmt, "\n"), $($arg)*));
}
