use crate::{drivers::vga::Vga13h, fall::FallingTetromino};

use super::{TetrisBoard, TetrisCell};

impl TetrisBoard {
	fn draw_cell(&self, fb: &mut [u8], state: TetrisCell, x: usize, y: usize) {
		let color = state.vga_color();
		for yo in 0..TetrisBoard::CELL_SIZE {
			let left = Vga13h::pix_idx(x, y + yo);
			let right = Vga13h::pix_idx(x + TetrisBoard::CELL_SIZE, y + yo);
			fb[left..right].copy_from_slice(&[color; TetrisBoard::CELL_SIZE]);
		}
	}

	fn draw_frame(&self, fb: &mut [u8], x: usize, y: usize) {
		let color = 0b1110_0000;
		for yo in 0..TetrisBoard::CELL_SIZE {
			let left = Vga13h::pix_idx(x, y + yo);
			let right = Vga13h::pix_idx(x + TetrisBoard::CELL_SIZE, y + yo);
			fb[left..right].copy_from_slice(&[color; TetrisBoard::CELL_SIZE]);
		}
	}

	pub fn draw(&self, vga: &mut Vga13h, falling: Option<&FallingTetromino>) {
		let fb = vga.get_fb();
		// Draw cells
		for bx in 0..TetrisBoard::BOARD_WIDTH {
			for by in 0..TetrisBoard::BOARD_HEIGHT {
				let cell = self.board[by * TetrisBoard::BOARD_WIDTH + bx];
				let dx = (bx + 1) * TetrisBoard::CELL_SIZE;
				let dy = (by + 1) * TetrisBoard::CELL_SIZE;

				self.draw_cell(fb, cell, dx, dy);
			}
		}

		if let Some(falling) = falling {
			for (x, y) in falling.tiles() {
				let x = x as usize;
				let y = y as usize;
				let cell = TetrisCell::Blue;
				let dx = (x + 1) * TetrisBoard::CELL_SIZE;
				let dy = (y + 1) * TetrisBoard::CELL_SIZE;

				self.draw_cell(fb, cell, dx, dy);
			}
		}

		// Draw frame
		for bx in 0..TetrisBoard::BOARD_WIDTH + 2 {
			self.draw_frame(fb, bx * TetrisBoard::CELL_SIZE, 0);
			self.draw_frame(
				fb,
				bx * TetrisBoard::CELL_SIZE,
				(TetrisBoard::BOARD_HEIGHT + 1) * TetrisBoard::CELL_SIZE,
			);
		}
		for by in 0..TetrisBoard::BOARD_HEIGHT + 2 {
			self.draw_frame(fb, 0, by * 9);
			self.draw_frame(
				fb,
				(TetrisBoard::BOARD_WIDTH + 1) * TetrisBoard::CELL_SIZE,
				by * TetrisBoard::CELL_SIZE,
			);
		}
	}
}
